// Generated by CoffeeScript 1.12.7
(function() {
  var FS, Q, _, buzJson, c, durations, mysql, mysqlConnect, pg, pgConnect, program, readQueriesFile, runMysqlQueries, runPgQueries, runQueries, runQuery, runScript;

  _ = require('lodash');

  c = require('@buzuli/color');

  Q = require('q');

  FS = require('fs');

  pg = require('pg');

  mysql = require('mysql');

  buzJson = require('@buzuli/json');

  program = require('commander');

  durations = require('durations');

  readQueriesFile = function(file, separator) {
    var deferred;
    if (separator == null) {
      separator = ';';
    }
    deferred = Q.defer();
    if (file == null) {
      deferred.resolve([]);
    } else {
      FS.readFile(file, {
        encoding: 'utf8'
      }, function(error, contents) {
        var queries, queriesString;
        if (error) {
          return deferred.reject(error);
        } else {
          try {
            queries = _(contents.split(separator)).filter(function(query) {
              if (query != null) {
                return true;
              } else {
                return false;
              }
            }).map(function(query) {
              return _.trim(query);
            }).filter(function(query) {
              return query.length > 0;
            }).map(function(query) {
              return query + ';';
            }).value();
            queriesString = _(queries).join('\n\n');
            console.log(c.yellow("=== " + (c.blue('Queries')) + " ========================\n"));
            console.log(queriesString + "\n");
            console.log(c.yellow("===================================="));
            return deferred.resolve(queries);
          } catch (error1) {
            error = error1;
            return deferred.reject(error);
          }
        }
      });
    }
    return deferred.promise;
  };

  mysqlConnect = function(mysqlConfig, config) {
    var connectWatch, connection, deferred;
    deferred = Q.defer();
    connectWatch = durations.stopwatch().start();
    connection = mysql.createConnection(mysqlConfig);
    connection.connect(function(error) {
      var context;
      if (error) {
        console.log("Error connecting to MySQL:", error, "\nStack:\n", error.stack);
        return deferred.reject(error);
      } else {
        if (!config.quiet) {
          console.log("Connected.");
        }
        context = {
          client: connection,
          done: function(callback) {
            return connection.end(function() {
              if (callback) {
                return callback();
              }
            });
          }
        };
        return deferred.resolve(context);
      }
    });
    return deferred.promise;
  };

  pgConnect = function(config) {
    var connectWatch, deferred, pgClient;
    deferred = Q.defer();
    connectWatch = durations.stopwatch().start();
    pgClient = new pg.Client(config);
    return pgClient.connect().then(function() {
      var context;
      connectWatch.stop();
      if (!config.quiet) {
        console.log("Connected.");
      }
      context = {
        client: pgClient,
        done: function(error) {
          if (error) {
            return Promise.reject(error);
          } else {
            return pgClient.end();
          }
        }
      };
      return context;
    })["catch"](function(error) {
      console.log("Error connecting to PostgreSQL:", error, "\nStack:\n", error.stack);
      throw error;
    });
  };

  runQuery = function(client, queries, scheme) {
    var deferred, query;
    deferred = Q.defer();
    if (_(queries).size() > 0) {
      query = _(queries).first();
      console.log((c.yellow('Running Query')) + " '" + query + "'");
      client.query(query, function(error, results) {
        var rows;
        if (error != null) {
          return deferred.reject(error);
        } else {
          rows = scheme === 'mysql' ? results : results.rows;
          console.log(c.green("Result:\n"), rows, "\n");
          return runQuery(client, _.tail(queries), scheme).then(function() {
            return deferred.resolve(0);
          });
        }
      });
    } else {
      deferred.resolve(0);
    }
    return deferred.promise;
  };

  runMysqlQueries = function(config, queries) {
    var myCfg;
    myCfg = {
      host: config.host,
      port: config.port,
      user: config.username,
      password: config.password,
      database: config.database
    };
    console.log("Connecting to database:\n'" + (buzJson(myCfg)) + "'");
    return mysqlConnect(myCfg, config).then(function(context) {
      context.watch = durations.stopwatch().start();
      console.log(c.blue("Ready to query MySQL.\n"));
      return runQuery(context.client, queries, config.scheme).then(function() {
        context.watch.stop();
        return console.log(c.blue("Closing connection. All queries took " + (c.orange(context.watch.format()))));
      })["finally"](function() {
        return context.done();
      });
    })["catch"](function(error) {
      return console.log("Error running queries: " + error + "\nStack:\n" + error.stack);
    });
  };

  runPgQueries = function(config, queries) {
    var cfg, database, host, password, port, uri, username;
    username = config.username, password = config.password, host = config.host, port = config.port, database = config.database;
    uri = "postgres://" + username + ":" + password + "@" + host + ":" + port + "/" + database;
    cfg = {
      connectionString: uri
    };
    console.log("Connecting to URI '" + (c.blue(uri)) + "'");
    return pgConnect(cfg).then(function(context) {
      context.watch = durations.stopwatch().start();
      console.log(c.blue("Ready to query PostgreSQL.\n"));
      return runQuery(context.client, queries, config.scheme).then(function() {
        context.watch.stop();
        return console.log(c.blue("Closing connection. All queries took " + (c.orange(context.watch.format()))));
      })["finally"](function() {
        return context.done();
      });
    })["catch"](function(error) {
      return console.log("Error running queries: " + error + "\nStack:\n" + error.stack);
    });
  };

  runQueries = function(partialConfig, queries) {
    var config, ref, ref1, ref2, ref3, ref4, ref5, ref6;
    config = {
      scheme: (ref = partialConfig.scheme) != null ? ref : 'postgres',
      host: (ref1 = partialConfig.host) != null ? ref1 : 'localhost',
      port: (ref2 = partialConfig.port) != null ? ref2 : 5432,
      username: (ref3 = partialConfig.username) != null ? ref3 : 'postgres',
      password: (ref4 = partialConfig.password) != null ? ref4 : 'postgres',
      database: (ref5 = partialConfig.database) != null ? ref5 : 'postgres',
      quiet: (ref6 = partialConfig.quiet) != null ? ref6 : false
    };
    switch (config.scheme) {
      case 'postgres':
        return runPgQueries(config, queries);
      case 'mysql':
        return runMysqlQueries(config, queries);
      default:
        return console.error("Invalid scheme. Valid values are 'postgres' and 'mysql'");
    }
  };

  runScript = function() {
    var partialConfig, queriesFile;
    program.usage('[options] <query_file>').option('-s, --scheme <scheme>', 'scheme for the connection can be "postgres" or "mysql" (default is "postgres")').option('-D, --database <database>', 'database (default is "postgres")').option('-h, --host <host>', 'the Postgres server\'s hostname (default is "localhost")').option('-p, --port <port>', "the Postgres server's port (default is 5432)", parseInt).option('-P, --password <password>', 'user password (default is "")').option('-q, --quiet', 'Silence non-error output (default is false)').option('-u, --username <username>', 'user name (default is "postgres")').parse(process.argv);
    queriesFile = _(program.args).first();
    partialConfig = {
      scheme: program.scheme,
      host: program.host,
      port: program.port,
      username: program.username,
      password: program.password,
      database: program.database,
      quiet: program.quiet
    };
    return readQueriesFile(queriesFile).then(function(queries) {
      return runQueries(partialConfig, queries);
    })["catch"](function(error) {
      return console.log("Error reading queries from file '" + queriesFile + "' : " + error + "\nStack:\n" + error.stack);
    });
  };

  module.exports = {
    runQueries: runQueries,
    run: runScript
  };

  if (require.main === module) {
    runScript();
  }

}).call(this);
